@@ -3,6 +3,7 @@ import urllib
 from urlparse import urlparse
 import base64
 import UserDict
+from contextlib import closing
 
 #class UJsonLibDecoder(object): # pragma: no cover
 #    def __init__(self):
@@ -29,6 +30,7 @@ from intermine.lists.listmanager import ListManager
 from intermine.errors import ServiceError, WebserviceError
 from intermine.results import InterMineURLOpener, ResultIterator
 from intermine import idresolution
+from intermine.decorators import requires_version
 
 """
 Webservice Interaction Routines for InterMine Webservices
@@ -180,6 +182,7 @@ class Service(object):
     QUERY_PATH             = '/query/results'
     LIST_ENRICHMENT_PATH   = '/list/enrichment'
     WIDGETS_PATH           = '/widgets'
+    SEARCH_PATH            = '/search'
     QUERY_LIST_UPLOAD_PATH = '/query/tolist'
     QUERY_LIST_APPEND_PATH = '/query/append/tolist'
     MODEL_PATH             = '/model'
@@ -196,6 +199,7 @@ class Service(object):
     SCHEME                 = 'http://'
     SERVICE_RESOLUTION_PATH = "/check/"
     IDS_PATH               = "/ids"
+    USERS_PATH             = "/users"
 
     def __init__(self, root,
             username=None, password=None, token=None,
@@ -427,6 +431,34 @@ class Service(object):
             self.templates[name] = t
         return t
 
+    def _get_json(self, path, payload = None):
+        with closing(self.opener.open(self.root + path, payload)) as resp:
+            data = json.loads(resp.read())
+            if data['error'] is not None:
+                raise ServiceError(data['error'])
+            return data
+
+    def search(self, term, **facets):
+        """
+        Perform an unstructured search by term
+        =======================================
+
+        This seach method performs a search of all objects
+        indexed by the service endpoint, returning results
+        and facets for those results.
+
+        @param term The search term
+        @param facets The facets to search by (eg: Organism = 'H. sapiens')
+
+        @return (list, dict) The results, and a dictionary of facetting informtation.
+        """
+        params = [('q', term)]
+        for facet, value in facets.iteritems():
+            params.append(("facet_" + facet, value))
+        payload = urllib.urlencode(params, doseq = True)
+        resp = self._get_json(self.SEARCH_PATH, payload = payload)
+        return (resp['results'], resp['facets'])
+
     @property
     def widgets(self):
         """
@@ -442,13 +474,8 @@ class Service(object):
         @return dict
         """
         if self._widgets is None:
-           sock = self.opener.open(self.root + self.WIDGETS_PATH)
-           text = sock.read()
-           sock.close()
-           data = json.loads(text)
-           if data['error'] is not None:
-               raise ServiceError(data['error'])
-           self._widgets = dict(([w['name'], w] for w in data['widgets']))
+            ws = self._get_json(self.WIDGETS_PATH)['widgets']
+            self._widgets = dict(([w['name'], w] for w in ws))
         return self._widgets
 
     def resolve_ids(self, data_type, identifiers, extra = '', case_sensitive = False, wildcards = False):
@@ -503,6 +530,8 @@ class Service(object):
         """
         Flushes any cached data.
         """
+        self._list_manager.delete_temporary_lists()
+        self._list_manager = ListManager(self)
         self._templates = None
         self._model = None
         self._version = None
@@ -591,3 +620,43 @@ class Service(object):
         """
         return ResultIterator(self, path, params, rowformat, view, cld)
 
+    @requires_version(9)
+    def register(self, username, password):
+        """
+        Register a new user with this service.
+        =======================================
+
+        @return {Service} an authenticated service.
+        """
+        payload = urllib.urlencode({'name': username, 'password': password})
+        registrar = Service(self.root)
+        resp = registrar._get_json(self.USERS_PATH, payload = payload)
+        token = resp['user']['temporaryToken']
+        return Service(self.root, token = token)
+
+    @requires_version(16)
+    def get_deregistration_token(self, validity = 300):
+        params = urllib.urlencode({'validity': validity})
+        resp = self._get_json('/user/deregistration', payload = params)
+        return resp['token']
+
+    @requires_version(16)
+    def deregister(self, deregistration_token):
+        """
+        Remove a User from the service
+        ==============================
+
+        @param deregistration_token A token to prove you really want to do this
+
+        @return string All the user's data.
+        """
+        if 'uuid' in deregistration_token:
+            deregistration_token = deregistration_token['uuid']
+
+        path = self.root + '/user'
+        params = {'deregistrationToken': deregistration_token, 'format': 'xml'}
+        uri = path + '?' + urllib.urlencode(params)
+        self.flush()
+        userdata = self.opener.delete(uri)
+        return userdata
+
